#ifndef RAWDCMCHAN_H
#define RAWDCMCHAN_H

#include "DAQDataFormats/RawDAQData.h"

#define RAWDCMCHAN_SIZE 1


namespace daqdataformats {

  namespace rawmillidcm {

        enum DCMChanMASKS{
            MASK_DiblockId   = 0x00FF, // Diblock ID
            MASK_DCMId       = 0x1F00, // DCM ID
            MASK_DetId       = 0xE000,  // Detector ID
            MASK_GlobalDCMID = 0xFFFF, // Diblock ID
        };
        enum DCMChanSHIFTS{
            SHIFT_DiblockId   = 0,
            SHIFT_DCMId       = 8,
            SHIFT_DetId       = 13,
            SHIFT_GlobalDCMID = 0,
        };

  }// end of  rawmillidcm
  
  class RawDCMChan : public RawDAQData {
    
  public:
    
    // Constructors
    RawDCMChan();                     //! The default constructor
    RawDCMChan(const RawDCMChan& copy_in);  //! Deep copy constructor
    RawDCMChan(void* dcmchan_buffer); //! Shallow copy constructor
    /// Destructor
    ~RawDCMChan();

    /// Initialization of the data members
    void init();

    /// Deep Copy
    void  copy(const RawDCMChan& copy_in);


    /// Print The DCM Channel identification information
    void print() const;

    // Getters
    uint32_t getDiblockId() const;
    uint32_t getDCMId() const;
    uint32_t getDetId() const;
    uint32_t getGlobalDCMID() const;
    // Setters
    bool setDiblockId    (uint32_t);
    bool setDCMId        (uint32_t);
    bool setDetId        (uint32_t);
    bool setGlobalDCMID  (uint32_t);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?


    /// Size of _Buffer in words
    uint32_t sizeofdata() const;

  private:
    
    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RAWDCMCHAN_SIZE];
    
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWDCMCHAN_H */
