#ifndef RAWEVENTHEADER_H
#define RAWEVENTHEADER_H
#include "DAQDataFormats/RawDAQData.h"


#define RAWEVENTHEADER_SIZE 8

namespace daqdataformats {

  namespace raweventhdr {

    // MilliSlice header delimiter. [0] value in the header, [1] value in the tail
    static const uint32_t Marker_value[2] = {0xE929, 0xE929};

        enum EventHeaderMASKS{
            MASK_MarkerRight         = 0x0000FFFF, //! Marker Mask left                  (16 bits)
            MASK_MarkerLeft          = 0xFFFF0000, //! Marker Mask right                 (16 bits)
            MASK_EventSize           = 0xFFFFFFFF, //! Size of the event in words        (8 bits)
            MASK_Version             = 0x00000FFF, //! Version Mask                      (12 bits)
            MASK_DataLoggerID        = 0xFFFF0000, //! DataLogger ID                     (16 bits)
            MASK_RunNumber           = 0x00FFFFFF, //! Run number                        (24 bits)
            MASK_RunType             = 0xFF000000, //! Run type                          (8 bits)
            MASK_ConfigIDX           = 0x0000FFFF, //! Detector configuration identifier (16 bits)
            MASK_Partition           = 0x00FF0000, //! Data logger partition             (8 bits)
            MASK_Subrun              = 0x3F000000, //! SubRun number                     (6 bits)
            MASK_EventNumberLo       = 0xFFFFFFFF, //! Universal event number lower  0-31 bits
            MASK_EventNumberHi       = 0x000000FF, //! Universal event number upper 32-39 bits
            MASK_EventType           = 0xFF000000, //! Size of the event in words        (8 bits)
            MASK_DataBlockNumber     = 0x0000FFFF, //! Number of event size              (16 bits)
            MASK_EventIncompleteFlag = 0x20000000, //! Event incomplete flag             (1 bit)
            MASK_MonteCarloFlag      = 0x40000000, //! SIM flag                          (1 bit)
            MASK_CRCCalculationUsed  = 0x80000000  //! Is CRC calculation used?          (1 bit)
        };


        enum EventHeaderSHIFTS{
            SHIFT_MarkerRight         = 0,
            SHIFT_MarkerLeft          = 16,
            SHIFT_EventSize           = 0,
            SHIFT_Version             = 0,
            SHIFT_DataLoggerID        = 16,
            SHIFT_RunNumber           = 0,
            SHIFT_RunType             = 24,
            SHIFT_ConfigIDX           = 0,
            SHIFT_Partition           = 16,
            SHIFT_Subrun              = 24,
            SHIFT_EventNumberLo       = 0,
            SHIFT_EventNumberHi       = 0,
            SHIFT_EventType           = 24,
            SHIFT_DataBlockNumber     = 0,
            SHIFT_EventIncompleteFlag = 29, //! Event incomplete flag             (1 bit)
            SHIFT_MonteCarloFlag      = 30,
            SHIFT_CRCCalculationUsed  = 31
        };

        enum EventHeaderWORDS{
            WORD_MarkerRight         = 0,
            WORD_MarkerLeft          = 0,
            WORD_EventSize           = 1,
            WORD_Version             = 2,
            WORD_DataLoggerID        = 2,
            WORD_RunNumber           = 3,
            WORD_RunType             = 3,
            WORD_ConfigIDX           = 4,
            WORD_Partition           = 4,
            WORD_Subrun              = 4,
            WORD_EventNumberLo       = 5,
            WORD_EventNumberHi       = 6,
            WORD_EventType           = 6,
            WORD_DataBlockNumber     = 7,
            WORD_EventIncompleteFlag = 7, //! Event incomplete flag             (1 bit)
            WORD_MonteCarloFlag      = 7,
            WORD_CRCCalculationUsed  = 7
        };

  } 
  
  class RawEventHeader: public RawDAQData {
    
  public:
    
    // Constructors
    RawEventHeader ();                         //! The default constructor
    RawEventHeader (const RawEventHeader& copy_in);  //! Deep copy constructor
    RawEventHeader (void* eventheader_buffer); //! Shallow copy constructor
    /// Destructor
    ~RawEventHeader();

    /// Initialization of the data members
    void init();


    /// Deep Copy
    void  copy(const RawEventHeader& copy_in);

    /// Print the MilliSlice Header
    void print() const;

    /// Getters
    uint32_t getMarkerLeft()        const;
    uint32_t getMarkerRight()       const;
    uint32_t getVersion()           const;
    uint32_t getDataLoggerID()      const;
    uint32_t getRunNumber()         const;
    uint32_t getRunType()           const;
    uint32_t getConfigIDX()         const;
    uint32_t getPartition()         const;
    uint32_t getSubrun()            const;
    uint64_t getEventNumber()       const;
    uint32_t getEventNumberLo()     const;
    uint32_t getEventNumberHi()     const;
    uint32_t getEventType()         const;
    uint32_t getEventSize()         const;
    uint32_t getDataBlockNumber()   const;
    bool     getMonteCarloFlag()    const;
    bool     isCRCCalculationUsed() const;
    bool     isEventIncomplete()    const;

    // Setters
    bool setMarkerLeft         (uint32_t);
    bool setMarkerRight        (uint32_t);
    bool setVersion            (uint32_t);
    bool setDataLoggerID       (uint32_t);
    bool setRunNumber          (uint32_t);
    bool setRunType            (uint32_t);
    bool setConfigIDX          (uint32_t);
    bool setPartition          (uint32_t);
    bool setSubrun             (uint32_t);
    bool setEventNumber        (uint64_t);
    bool setEventNumberLo      (uint32_t);
    bool setEventNumberHi      (uint32_t);
    bool setEventType          (uint32_t);
    bool setEventSize          (uint32_t);
    bool addEventSize          (uint32_t size);
    bool setDataBlockNumber    (uint32_t);
    bool setMonteCarloFlag     (bool);
    bool setCRCCalculationUsed (bool);
    bool setEventIncomplete    (bool);
    bool advanceDataBlockNumber();
    bool advanceEventNumber    ();

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?

    /// MilliSlice Header size in words
    uint32_t sizeofdata() const;

  private:

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RAWEVENTHEADER_SIZE];

  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWEVENTHEADER_H */
