#ifndef RAWEVENTTAIL_H
#define RAWEVENTTAIL_H
#include "DAQDataFormats/RawDAQData.h"


#define RAWEVENTTAIL_SIZE 4

namespace daqdataformats {

  namespace raweventtail {

    // MilliSlice header delimiter. [0] value in the header, [1] value in the tail
    static const uint32_t Marker_value[3] = {0xEEEE, 0x929E, 0x929E};

        enum EventTailMASKS{
            MASK_EventNumberHi = 0x000000FF,//! Universal event number lower  32-39 bits
            MASK_Marker        = 0xFFFF0000,// Marker Mask                 (16 bits)
            MASK_EventNumberLo = 0xFFFFFFFF,//! Universal event number lower  0-31 bits
            MASK_EventSize     = 0xFFFFFFFF, //! Size of the event in words  (8 bits)
            MASK_MarkerRight   = 0x0000FFFF,// Marker Mask                 (16 bits)
            MASK_MarkerLeft    = 0xFFFF0000// Marker Mask                 (16 bits)
        };


        enum EventTailSHIFTS{
            SHIFT_EventNumberHi = 0,
            SHIFT_Marker        = 16,
            SHIFT_EventNumberLo = 0,
            SHIFT_EventSize     = 0,
            SHIFT_MarkerRight   = 0,
            SHIFT_MarkerLeft    = 16
        };

        enum EventTailWORDS{
            WORD_EventNumberHi = 0,
            WORD_Marker        = 0,
            WORD_EventNumberLo = 1,
            WORD_EventSize     = 2,
            WORD_MarkerRight   = 3,
            WORD_MarkerLeft    = 3
        };

  } 
  
  class RawEventTail: public RawDAQData {
    
  public:
    
    // Constructors
    RawEventTail ();                       //! The default constructor
    RawEventTail (RawEventTail& copy_in);  //! Deep copy constructor
    RawEventTail (RawEventTail* copy_in);  //! Deep copy constructor
    RawEventTail (void* eventtail_buffer); //! Shallow copy constructor
    /// Destructor
    ~RawEventTail();

    /// Initialization of the data members
    void init();


    /// Deep Copy
    void  copy(RawEventTail& copy_in);
    void  copy(RawEventTail* copy_in);
    void  copy(void* copy_in);

    /// Print the MilliSlice Header
    void print() const;

    /// Getters
    uint32_t getMarker() const;
    uint32_t getMarkerLeft() const;
    uint32_t getMarkerRight() const;
    uint32_t getEventNumberLo() const;
    uint32_t getEventNumberHi() const;
    uint32_t getEventSize() const;

    // Setters
    bool setMarker        (uint32_t);
    bool setMarkerLeft    (uint32_t);
    bool setMarkerRight   (uint32_t);
    bool setEventNumberLo (uint32_t);
    bool setEventNumberHi (uint32_t);
    bool setEventSize     (uint32_t);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?

    /// MilliSlice Header size in words
    uint32_t sizeofdata() const;

  private:
    

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RAWEVENTTAIL_SIZE];

  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWEVENTTAIL_H */
