#ifndef RAWMICROBLOCK_H
#define RAWMICROBLOCK_H

#include "DAQDataFormats/RawMicroSlice.h"
#include "DAQDataFormats/RawMicroBlockHeader.h"

namespace daqdataformats {

    #define DEFAULT_MICROBLOCK_SIZE 16385 /* Default MicroBlock Buffer Size  */

  class RawMicroBlock : public RawDAQData {

  public:
    // Constructors
    RawMicroBlock (uint32_t buffersize = DEFAULT_MICROBLOCK_SIZE); //! The default constructor
    RawMicroBlock (const RawMicroBlock& copy_in);                  //! Deep copy constructor
    RawMicroBlock (void* microblock_buffer, uint32_t buffersize = DEFAULT_MICROBLOCK_SIZE); //! Shallow copy constructor
    /// Destructor
    ~RawMicroBlock();

    /// Initialization of Data members
    void init();

    /// Deep Copy
    void copy(const RawMicroBlock& copy_in);

    
    /// Read method is for reading the whole RawMicroBlock
    void* readData(void* millislice_buffer);
    // Suplementary reading methods
    void* readData(void* buffer, uint32_t size);
    void* readData(void* buffer, uint32_t size, uint32_t _Buffer_start_position);

    /// Size of MicroBlock _Buffer in words
    uint32_t sizeofdata() const;

    /// Print the MicroBlock structure
    void print() ;

    // Getters
    RawMicroBlockHeader* getHeader();            //! MicroBlock Header reference
    RawMicroSlice*       getMicroSlice();        //! The current floating MicroSlice reference

    // Setters
    bool setMicroBlockHeader(void* millislice_buffer);// Input is the pointer to the MilliSlice(or equivalently MilliSliceHeader)!
    bool setMicroSlice      (void* microslice_buffer);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;        //!< Is the buffer internal?

  private:

    RawMicroBlockHeader     _Header;          //! MicroBlock Header
    RawMicroSlice           _MicroSlice;      //! MicroSlice

   //! Maximum size of the current buffer
    uint32_t                _max_buffer_size;
    // The actual internal buffer (not a pointer).
    // Used only if the constructors with the default size are used
    std::vector <uint32_t>  _InternalBuffer;


  };


} // END NAMESPCE daqdataformats
#endif /* RAWMICROBLOCK_H */
