#ifndef RAWMILLISLICEINDEX_H
#define RAWMILLISLICEINDEX_H

#include "DAQDataFormats/RawDCMChan.h"

#include <vector>


#define DEFAULT_MILLISLICEINDEX_MAXIMUM_SIZE 2000
#define MILLISLICEINDEX_SIZE 2 //! The size of one index

namespace daqdataformats {

    namespace rawmilliindx {


        enum MilliSliceIndexMASKS{
            MASK_Time     = 0xFFFFFFFF, // Time length since the MilliSlice start time
            MASK_Offset   = 0x000FFFFF  // Offset
        };

        enum MilliSliceIndexSHIFTS{
            SHIFT_Time      = 0,
            SHIFT_Offset    = 0
        };

        enum MilliSliceIndexWORDS{
            WORD_Time       = 0,
            WORD_Offset     = 1
        };
    }// end of namespace rawmilliindx

  
  class RawMilliSliceIndex: public RawDAQData {
    
  public:
    
    // Constructors
    RawMilliSliceIndex(uint32_t size = DEFAULT_MILLISLICEINDEX_MAXIMUM_SIZE);             //! The default constructor
    RawMilliSliceIndex(const RawMilliSliceIndex& copy_in); //! Deep copy constructor
    RawMilliSliceIndex(void* millisliceheader_buffer, uint32_t size = DEFAULT_MILLISLICEINDEX_MAXIMUM_SIZE, uint32_t indexnumber = 0); //! Shallow copy constructor
    /// Destructor
    ~RawMilliSliceIndex();

    /// Initialization of the data members
    void init();

    /// Deep Copy
    void  copy(const RawMilliSliceIndex& copy_in);

    /// Print the MilliSlice Header
    void print() const;

    /// Getters
    uint32_t getTime       (uint32_t iindex) const; //! Get the Time length of the iindex-th MilliSliceIndex
    uint32_t getOffset     (uint32_t iindex) const; //! Get the Offset of the iindex-th MilliSliceIndex
    uint32_t getIndexNumber() const;                //! Get the current number of indeces in the MilliSliceIndex

    bool addIndex          (uint32_t time_length, uint32_t offset);
    bool setIndexNumber    (uint32_t);
    bool advanceIndexNumber();
    bool setTime           (uint32_t iindex, uint32_t time_length);
    bool setOffset         (uint32_t iindex, uint32_t offset);

    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;        //!< Is the buffer internal?

    /// MilliSlice Header size in words
    uint32_t sizeofdata() const;

  private:
    //! Maximum size of the current buffer (used for external buffers)
    uint32_t               _max_buffer_size;

    uint32_t               _IndexNumber;

    /// The actual internal buffer (not a pointer)
    std::vector <uint32_t> _InternalBuffer;


  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWMILLISLICEINDEX_H */
