#ifndef RAWRUNHEADER_H
#define RAWRUNHEADER_H

#include "DAQDataFormats/RawDAQData.h"

#define RUNHEADER_SIZE 25

namespace daqdataformats {

    namespace rawruntail {

        // Run tail delimiters
        static const uint32_t Marker_value[4] = {0x4C494154, 0x4C494154, 0x39323945, 0x41764F4E};
    }

    namespace rawrunhdr {

        // Run header delimiter
        static const uint32_t Marker_value[2] = {0x41764F4E, 0x39323945};

        enum RunHeaderMASKS{
            MASK_MarkerHi                 = 0xFFFFFFFF, //! Marker Mask upper word (32 bits)
            MASK_MarkerLo                 = 0xFFFFFFFF, //! Marker Mask lower word (32 bits)
            MASK_RunNumber                = 0x00FFFFFF, //! Run number             (24 bits)
            MASK_RunType                  = 0xFF000000, //! Run type               (8 bits)
            MASK_ConfigIDX                = 0x0000FFFF, //! Detector configuration identifier (16 bits)
            MASK_Partition                = 0x00FF0000, //! Partition identifier that Run Control and Data Logger belong to (8 bits)
            MASK_Subrun                   = 0x3F000000, //! Subrun number          (6 bits)
            MASK_ConfigurationBlockSet    = 0x80000000, //! Is the configuration Block set? (1bit)
            MASK_RunControlVersion        = 0x00000FFF, //! Run Control version    (12 bits)
            MASK_RunControlID             = 0xFFFF0000, //! Run control identifier (16 bits)
            MASK_DataLoggerVersion        = 0x00000FFF, //! Data Logger version    (12 bits)
            MASK_DataLoggerID             = 0xFFFF0000, //! Data Logger identifier (16 bits)
            MASK_RunStartTime             = 0xFFFFFFFF, //! Run Start Time         (32 bits)
            MASK_SubRunStartTime          = 0xFFFFFFFF, //! SubRun Start Time      (32 bits)
            MASK_SubRunEndTime            = 0xFFFFFFFF, //! SubRun End Time        (32 bits)
            MASK_RunEndTime               = 0xFFFFFFFF, //! Run End Time           (32 bits)
            MASK_TotalEvents              = 0xFFFFFFFF, //! Total number of events (32 bits)
            MASK_UniversalEventStartLo    = 0xFFFFFFFF, //! Universal event number lower 32 bits
            MASK_UniversalEventStartHi    = 0x000000FF, //! Universal event number higher 8 bits
            MASK_UniversalEventEndLo      = 0xFFFFFFFF, //! Universal event number lower 32 bits
            MASK_UniversalEventEndHi      = 0x000000FF, //! Universal event number higher 8 bits
            MASK_RunSize                  = 0xFFFFFFFF, //! Size of the run in words
            MASK_TriggerVersion           = 0x00000FFF, //! Trigger version (12 bits)
            MASK_TriggerCtrlID            = 0xFFFF0000, //! Identifier of the global trigger processor (16 bits)
            MASK_TriggerMC                = 0x80000000, //! MC triggering flag
            MASK_VTMod                    = 0x3F000000, //! Valid trigger modification modes
            MASK_ValidTriggerTypesLow     = 0x00FFFFFF, //! Logical OR of all valid trigger types. First word
            MASK_ValidTriggerTypesHigh    = 0xFFFFFFFF, //! Logical OR of all valid trigger types. Second word
            MASK_ValidTriggerTypesHigh2   = 0xFFFFFFFF, //! Logical OR of all valid trigger types. Third word
            MASK_TriggerListIDX           = 0xFFFFFFFF,
            MASK_TriggerPrescaleListIDX   = 0xFFFFFFFF,
            MASK_Reserved1                = 0xFFFFFFFF,
            MASK_Reserved2                = 0xFFFFFFFF,
            MASK_CRC                      = 0xFFFFFFFF

        };

        enum RunHeaderSHIFTS{
            SHIFT_MarkerHi                 = 0,  //! Marker Mask upper word (32 bits)
            SHIFT_MarkerLo                 = 0,  //! Marker Mask lower word (32 bits)
            SHIFT_RunNumber                = 0,  //! Run number             (24 bits)
            SHIFT_RunType                  = 24, //! Run type               (8 bits)
            SHIFT_ConfigIDX                = 0,  //! Detector configuration identifier (16 bits)
            SHIFT_Partition                = 16, //! Partition identifier that Run Control and Data Logger belong to (8 bits)
            SHIFT_Subrun                   = 24, //! Subrun number          (6 bits)
            SHIFT_ConfigurationBlockSet    = 31, //! Is the configuration Block set? (1bit)
            SHIFT_RunControlVersion        = 0, //! Run Control version    (12 bits)
            SHIFT_RunControlID             = 16, //! Run control identifier (16 bits)
            SHIFT_DataLoggerVersion        = 0, //! Data Logger version    (12 bits)
            SHIFT_DataLoggerID             = 16, //! Data Logger identifier (16 bits)
            SHIFT_RunStartTime             = 0, //! Run Start Time         (32 bits)
            SHIFT_SubRunStartTime          = 0, //! SubRun Start Time      (32 bits)
            SHIFT_SubRunEndTime            = 0, //! SubRun End Time        (32 bits)
            SHIFT_RunEndTime               = 0, //! Run End Time           (32 bits)
            SHIFT_TotalEvents              = 0, //! Total number of events (32 bits)
            SHIFT_UniversalEventStartLo    = 0, //! Universal event number lower 32 bits
            SHIFT_UniversalEventStartHi    = 0, //! Universal event number higher 8 bits
            SHIFT_UniversalEventEndLo      = 0, //! Universal event number lower 32 bits
            SHIFT_UniversalEventEndHi      = 0, //! Universal event number higher 8 bits
            SHIFT_RunSize                  = 0, //! Size of the run in words
            SHIFT_TriggerVersion           = 0, //! Trigger version (12 bits)
            SHIFT_TriggerCtrlID            = 16,//! Identifier of the global trigger processor (16 bits)
            SHIFT_TriggerMC                = 31,//! MC triggering flag
            SHIFT_VTMod                    = 24,//! Valid trigger modification modes
            SHIFT_ValidTriggerTypesLow     = 0, //! Logical OR of all valid trigger types. First word
            SHIFT_ValidTriggerTypesHigh    = 0, //! Logical OR of all valid trigger types. Second word
            SHIFT_ValidTriggerTypesHigh2   = 0, //! Logical OR of all valid trigger types. Third word
            SHIFT_TriggerListIDX           = 0,
            SHIFT_TriggerPrescaleListIDX   = 0,
            SHIFT_Reserved1                = 0,
            SHIFT_Reserved2                = 0,
            SHIFT_CRC                      = 0
        };

        enum RunHeaderWORDS{
            WORD_MarkerHi                 = 0,  //! Marker Mask upper word (32 bits)
            WORD_MarkerLo                 = 1,  //! Marker Mask lower word (32 bits)
            WORD_RunNumber                = 2,  //! Run number             (24 bits)
            WORD_RunType                  = 2, //! Run type               (8 bits)
            WORD_ConfigIDX                = 3,  //! Detector configuration identifier (16 bits)
            WORD_Partition                = 3, //! Partition identifier that Run Control and Data Logger belong to (8 bits)
            WORD_Subrun                   = 3, //! Subrun number          (6 bits)
            WORD_ConfigurationBlockSet    = 3, //! Is the configuration Block set? (1bit)
            WORD_RunControlVersion        = 4, //! Run Control version    (12 bits)
            WORD_RunControlID             = 4, //! Run control identifier (16 bits)
            WORD_DataLoggerVersion        = 5, //! Data Logger version    (12 bits)
            WORD_DataLoggerID             = 5, //! Data Logger identifier (16 bits)
            WORD_RunStartTime             = 6, //! Run Start Time         (32 bits)
            WORD_SubRunStartTime          = 7, //! SubRun Start Time      (32 bits)
            WORD_SubRunEndTime            = 8, //! SubRun End Time        (32 bits)
            WORD_RunEndTime               = 9, //! Run End Time           (32 bits)
            WORD_TotalEvents              = 10, //! Total number of events (32 bits)
            WORD_UniversalEventStartLo    = 11, //! Universal event number lower 32 bits
            WORD_UniversalEventStartHi    = 12, //! Universal event number higher 8 bits
            WORD_UniversalEventEndLo      = 13, //! Universal event number lower 32 bits
            WORD_UniversalEventEndHi      = 14, //! Universal event number higher 8 bits
            WORD_RunSize                  = 15, //! Size of the run in words
            WORD_TriggerVersion           = 16, //! Trigger version (12 bits)
            WORD_TriggerCtrlID            = 16,//! Identifier of the global trigger processor (16 bits)
            WORD_TriggerMC                = 17,//! MC triggering flag
            WORD_VTMod                    = 17,//! Valid trigger modification modes
            WORD_ValidTriggerTypesLow     = 17, //! Logical OR of all valid trigger types. First word
            WORD_ValidTriggerTypesHigh    = 18, //! Logical OR of all valid trigger types. Second word
            WORD_ValidTriggerTypesHigh2   = 19, //! Logical OR of all valid trigger types. Third word
            WORD_TriggerListIDX           = 20,
            WORD_TriggerPrescaleListIDX   = 21,
            WORD_Reserved1                = 22,
            WORD_Reserved2                = 23,
            WORD_CRC                      = 24
        };
    }// end of namespace rawrunhdr

  
  class RawRunHeader: public RawDAQData {
    
  public:
    
    // Constructors
    RawRunHeader();                              //! The default constructor
    RawRunHeader(const RawRunHeader& copy_in);  //! Deep copy constructor
    RawRunHeader(void* millisliceheader_buffer); //! Shallow copy constructor
    /// Destructor
    ~RawRunHeader();

    /// Initialization of the data members
    void init();

    /// Deep Copy
    void  copy(const RawRunHeader& copy_in);

    /// Print the Run Header
    void print() const;


    // Closing the Run Header. Calculating the CRC
    bool close();

    /// Getters
    uint32_t getMarkerHi()               const;
    uint32_t getMarkerLo()               const;
    uint32_t getRunNumber()              const;
    uint32_t getRunType()                const;
    uint32_t getConfigIDX()              const;
    uint32_t getPartition()              const;
    uint32_t getSubrun()                 const;
    uint32_t getConfigurationBlockSet()  const;
    uint32_t getRunControlVersion()      const;
    uint32_t getRunControlID()           const;
    uint32_t getDataLoggerVersion()      const;
    uint32_t getDataLoggerID()           const;
    uint32_t getRunStartTime()           const;
    uint32_t getSubRunStartTime()        const;
    uint32_t getSubRunEndTime ()         const;
    uint32_t getRunEndTime()             const;
    uint32_t getTotalEvents()            const;
    uint32_t getUniversalEventStartLo()  const;
    uint32_t getUniversalEventStartHi()  const;
    uint64_t getUniversalEventStart()    const;
    uint32_t getUniversalEventEndLo()    const;
    uint32_t getUniversalEventEndHi()    const;
    uint64_t getUniversalEventEnd()      const;
    uint32_t getRunSize()                const;
    uint32_t getTriggerVersion()         const;
    uint32_t getTriggerCtrlID()          const;
    uint32_t getTriggerMC()              const;
    uint32_t getVTMod()                  const;
    uint32_t getValidTriggerTypesLow()   const;
    uint32_t getValidTriggerTypesHigh()  const;
    uint32_t getValidTriggerTypesHigh2() const;
    uint32_t getTriggerListIDX()         const;
    uint32_t getTriggerPrescaleListIDX() const;
    uint32_t getReserved1()              const;
    uint32_t getReserved2()              const;


// Setters
    bool setMarkerHi              (uint32_t);
    bool setMarkerLo              (uint32_t);
    bool setRunNumber             (uint32_t);
    bool setRunType               (uint32_t);
    bool setConfigIDX             (uint32_t);
    bool setPartition             (uint32_t);
    bool setSubrun                (uint32_t);
    bool setConfigurationBlockSet (uint32_t);
    bool setRunControlVersion     (uint32_t);
    bool setRunControlID          (uint32_t);
    bool setDataLoggerVersion     (uint32_t);
    bool setDataLoggerID          (uint32_t);
    bool setRunStartTime          (uint32_t);
    bool setSubRunStartTime       (uint32_t);
    bool setSubRunEndTime         (uint32_t);
    bool setRunEndTime            (uint32_t);
    bool setTotalEvents           (uint32_t);
    bool setUniversalEventStartLo (uint32_t);
    bool setUniversalEventStartHi (uint32_t);
    bool setUniversalEventStart   (uint64_t);
    bool setUniversalEventEndLo   (uint32_t);
    bool setUniversalEventEndHi   (uint32_t);
    bool setUniversalEventEnd     (uint64_t);
    bool setRunSize               (uint32_t);
    bool setTriggerVersion        (uint32_t);
    bool setTriggerCtrlID         (uint32_t);
    bool setTriggerMC             (uint32_t);
    bool setVTMod                 (uint32_t);
    bool setValidTriggerTypesLow  (uint32_t);
    bool setValidTriggerTypesHigh (uint32_t);
    bool setValidTriggerTypesHigh2(uint32_t);
    bool setTriggerListIDX        (uint32_t);
    bool setTriggerPrescaleListIDX(uint32_t);
    bool setReserved1             (uint32_t);
    bool setReserved2             (uint32_t);
    bool setTail(); /// Set this thin to Tail


    // Adding the trigger type using OR
    bool addValidTriggerTypesLow  (uint32_t);
    bool addValidTriggerTypesHigh (uint32_t);
    bool addValidTriggerTypesHigh2(uint32_t);
    bool addRunSize               (uint32_t size);
    bool advanceEventNumber();

    
    bool setBufferSource (void* pointer); //!< Set The Buffer pointer to a given location. Don't use it if you are not sure
    bool resetBufferSource();             //!< Reset the class to use its internal buffer
    bool clear();                         //!< Clear the buffer
    bool reset();                         //!< Reset the class data
    bool isInternalBuffer() const;              //!< Is the buffer internal?


    /// Run Header size in words
    uint32_t sizeofdata() const;

  private:

    /// The actual internal buffer (not a pointer)
    uint32_t  _InternalBuffer[RUNHEADER_SIZE];
  };
  
  
} // END NAMESPCE daqdataformats
#endif /* RAWRUNHEADER_H */
