#include "DAQDataFormats/RawConfigurationNamespace.h"
#include <sys/time.h>		// gettimeofday
#include <boost/lexical_cast.hpp>
#include <stdio.h>


#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif

using namespace daqdataformats;
using namespace rawconftail;

//------------------------------------------------------------------------------
// Default constructor
RawConfigurationNamespace::RawConfigurationNamespace(uint32_t tlvl) : _tlvl(tlvl)
{
    tprintf( 24,"RawConfigurationNamespace(uint32_t size) construct (this=%p)\n",this );

    _InternalBuffer.clear();
}

//------------------------------------------------------------------------------
RawConfigurationNamespace::RawConfigurationNamespace(const RawConfigurationNamespace& copy_in){
    this->copy(copy_in);
}


//------------------------------------------------------------------------------
// Destructor
RawConfigurationNamespace::~RawConfigurationNamespace() { tprintf(24,"~RawConfigurationNamespace (this=%p)\n",this);
}

//------------------------------------------------------------------------------
// General initialization
void RawConfigurationNamespace::init(){
    _fd_read_bytes = 0;
}

//------------------------------------------------------------------------------
void RawConfigurationNamespace::copy(const RawConfigurationNamespace& copy_in){


    long unsigned int size = copy_in._InternalBuffer.size();
    _InternalBuffer.reserve(size);
    _Buffer = (void*)&_InternalBuffer[0];



    /// Copy the data from one buffer to the other
    for(uint32_t i = 0; i < size; ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

}


//------------------------------------------------------------------------------
// Setters
bool RawConfigurationNamespace::setNamespace(std::string& namespace_name){


    std::cout<<"Setting to "<<namespace_name<<"\n";

    long unsigned int size = (long unsigned int)namespace_name.size() / 4;

    if(namespace_name.size()%4)size++;

    /// Resize the Internal buffer
    _InternalBuffer.resize(size);
    _Buffer = (void*)&_InternalBuffer[0];


    /// counter for the char
    uint32_t ichar = 0;

    /// Fill the Namespace fields
    for(uint32_t iword = 0        ; iword < size; ++iword)
    for(uint32_t ichar_in_word = 0; ichar_in_word < 4; ++ichar_in_word){


        /// If current char is still one of the characters of the string
        /// that we are setting it to, read it
        /// Otherwise, make it zero.
        uint32_t letter;
        if(ichar < namespace_name.size())
            letter = namespace_name[ichar];
        else
            letter = 0;
        
        /// set the char
        switch(ichar_in_word){
        case 0:
            this->setChar1(letter, iword);
        case 1:
            this->setChar2(letter, iword);
        case 2:
            this->setChar3(letter, iword);
        case 3:
            this->setChar4(letter, iword);
        }// end of cases

        ++ichar;
    }
        

    return true;
}

// Setters for Chars
bool RawConfigurationNamespace::setChar1(uint32_t letter, uint32_t word_number){
    BITFIELDSET(_InternalBuffer[word_number], letter, MASK_Char1, SHIFT_Char1);
    return true;
}
bool RawConfigurationNamespace::setChar2(uint32_t letter, uint32_t word_number){
    BITFIELDSET(_InternalBuffer[word_number], letter, MASK_Char2, SHIFT_Char2);
    return true;
}
bool RawConfigurationNamespace::setChar3(uint32_t letter, uint32_t word_number){
    BITFIELDSET(_InternalBuffer[word_number], letter, MASK_Char3, SHIFT_Char3);
    return true;
}
bool RawConfigurationNamespace::setChar4(uint32_t letter, uint32_t word_number){
    BITFIELDSET(_InternalBuffer[word_number], letter, MASK_Char4, SHIFT_Char4);
    return true;
}


//------------------------------------------------------------------------------
// Getters
uint32_t RawConfigurationNamespace::getNamespace(std::string& namespace_name) const{

    namespace_name = "";


    /// Fill the Namespace fields
    for(uint32_t iword = 0        ; iword < this->sizeofdata(); ++iword)
    for(uint32_t ichar_in_word = 0; ichar_in_word < 4; ++ichar_in_word){

        if      (ichar_in_word == 0 )
            namespace_name += (char)this->getChar1(iword);
        else if (ichar_in_word == 1 )
            namespace_name += (char)this->getChar2(iword);
        else if (ichar_in_word == 2 )
            namespace_name += (char)this->getChar3(iword);
        else if (ichar_in_word == 3 )
            namespace_name += (char)this->getChar4(iword);

    }

    return this->sizeofdata();
}

// Getters for Chars
uint32_t RawConfigurationNamespace::getChar1(uint32_t word_number) const {
    return BITFIELDGET((_InternalBuffer[word_number]), MASK_Char1, SHIFT_Char1);
}
// Getters for Chars
uint32_t RawConfigurationNamespace::getChar2(uint32_t word_number) const {
    return BITFIELDGET(_InternalBuffer[word_number], MASK_Char2, SHIFT_Char2);
}
// Getters for Chars
uint32_t RawConfigurationNamespace::getChar3(uint32_t word_number) const {
    return BITFIELDGET(_InternalBuffer[word_number], MASK_Char3, SHIFT_Char3);
}
// Getters for Chars
uint32_t RawConfigurationNamespace::getChar4(uint32_t word_number) const {
    return BITFIELDGET(_InternalBuffer[word_number], MASK_Char4, SHIFT_Char4);
}




//------------------------------------------------------------------------------
// Read an entire DataBlock from datablock_buffer into _Buffer
void* RawConfigurationNamespace::readData(void* datablock_buffer){
    
    std::cerr<<"\nCannot read the RawConfigurationNamespace without the given size";
    
    return 0;
}

//------------------------------------------------------------------------------
void* RawConfigurationNamespace::readData(void* void_buffer, uint32_t size){

  uint32_t* buffer = (uint32_t*)void_buffer;

  if(buffer == 0){return 0;}


  /// Resize the internal buffer
  _InternalBuffer.resize((long unsigned int)size);
  _Buffer = (void*)&_InternalBuffer[0];

  /// Copy the data from one buffer to the other
  for(uint32_t i = 0; i < size; ++i)
        _InternalBuffer[i] = buffer[i];

  /// Return the address of the buffer advanced by size words;
  return (void*)(buffer + size);
}

//------------------------------------------------------------------------------
void* RawConfigurationNamespace::readData(void* buffer_void, uint32_t size, uint32_t _Buffer_start_position){

  _Buffer = (void*)&_InternalBuffer[0];

  uint32_t* buffer = (uint32_t*)buffer_void;

  if(buffer == 0){return 0;}

  for(uint32_t i = 0; i < size; ++i )
    RAWBUFF32[i+_Buffer_start_position] = buffer[i];

  // Return the address of the buffer advanced by 1 word;
  return (void*)(buffer+size);
}


//------------------------------------------------------------------------------
// Print the DataBlock structure
void RawConfigurationNamespace::print() const {

    printf("\n---RawConfigurationNamespace----------------------------------------------------size=%u\n",this->sizeofdata());

    std::string namespace_name;
    this->getNamespace(namespace_name);
    std::cout<<"Namespace = "<<namespace_name<<"\n";

}

//------------------------------------------------------------------------------
// Size of the _Buffer in words
uint32_t RawConfigurationNamespace::sizeofdata()const { return (uint32_t)_InternalBuffer.size();}

void
RawConfigurationNamespace::tprintf( unsigned lvl, const char *fmt, ... )
{   if (lvl > 31) lvl=31;
    if (_tlvl & (1<<lvl))
    {   timeval 	tv;
	char            str[80];
	time_t          local;
	const char*     indent="                                ";//32 spaces
	va_list         ap;
	va_start( ap, fmt);
	gettimeofday( &tv, NULL );
	local = tv.tv_sec;
	strftime( str, sizeof(str), "%T", localtime(&local) );
	printf( "%s.%06ld%s", str, (long)tv.tv_usec, &indent[31-lvl] );
	vprintf( fmt, ap );
    }
}   // tprintf
