#include "DAQDataFormats/RawDataBlockHeader.h"
#include <cstdio>

#ifndef RAWBUFF32
// Define a macros to make accessing the raw buffer easier
#define RAWBUFF32 ((uint32_t*)this->_Buffer)
#define RAWBUFF16 ((uint16_t*)this->_Buffer)
#endif


using namespace daqdataformats;
using namespace datablock;

//------------------------------------------------------------------------------
// Default constructor
RawDataBlockHeader::RawDataBlockHeader(){
    // Set pointers to point to the internal buffer
    this->resetBufferSource();

    // Initialize it
    this->init();
}

//------------------------------------------------------------------------------
RawDataBlockHeader::RawDataBlockHeader(const RawDataBlockHeader& copy_in){
    this->copy(copy_in);
}

//------------------------------------------------------------------------------
RawDataBlockHeader::RawDataBlockHeader(void* datablockheader_buffer){

    // First, initialize the Internal Buffer
            // Set pointers to point to the internal buffer
            this->resetBufferSource();

            // Initialize it
            this->init();

    // Now set the pointer to the external buffer
    this->setBufferSource(datablockheader_buffer);
}

//------------------------------------------------------------------------------
// Destructor
RawDataBlockHeader::~RawDataBlockHeader() { }

//------------------------------------------------------------------------------
// Initialization of data members
void RawDataBlockHeader::init(){
    // Zero out the whole buffer
    for(uint32_t i = 0; i < this->sizeofdata(); i++)
        RAWBUFF32[i] = 0;
    // Set the Marker
    this->setMarker(DataBlock_Marker);
    this->setDataSize(RAWDATABLOCKHEADER_SIZE);

}

//------------------------------------------------------------------------------
void RawDataBlockHeader::copy(const RawDataBlockHeader& copy_in){

    // Copy the data from one buffer to the other
    for(int i = 0; i < RAWDATABLOCKHEADER_SIZE; ++i)
        _InternalBuffer[i] = copy_in._InternalBuffer[i];

    // Setup the main buffer pointer
    // If the copy_in's buffer is external, point to that external memory
    if(copy_in.isInternalBuffer()) this->resetBufferSource();
    else                           this->setBufferSource(copy_in._Buffer);
}


//------------------------------------------------------------------------------
// Getters
      uint32_t RawDataBlockHeader::getVersion() const        { return BITFIELDGET(RAWBUFF32[WORD_Version]        , MASK_Version        , SHIFT_Version);}
      uint32_t RawDataBlockHeader::getMarker() const         { return BITFIELDGET(RAWBUFF32[WORD_Marker]         , MASK_Marker         , SHIFT_Marker); }
      uint32_t RawDataBlockHeader::getTrigNumLo() const      { return BITFIELDGET(RAWBUFF32[WORD_TriggerNumberLo], MASK_TriggerNumberLo, SHIFT_TriggerNumberLo); }
      uint32_t RawDataBlockHeader::getTrigNumHi() const      { return BITFIELDGET(RAWBUFF32[WORD_TriggerNumberHi], MASK_TriggerNumberHi, SHIFT_TriggerNumberHi); }
      uint16_t RawDataBlockHeader::getNumMicroBlocks() const { return BITFIELDGET(RAWBUFF32[WORD_NumMicroBlocks] , MASK_NumMicroBlocks , SHIFT_NumMicroBlocks); }
      uint16_t RawDataBlockHeader::getBuffId() const         { return BITFIELDGET(RAWBUFF32[WORD_BuffId]         , MASK_BuffId         , SHIFT_BuffId);}
      uint32_t RawDataBlockHeader::getDataSize() const       { return BITFIELDGET(RAWBUFF32[WORD_DataSize]       , MASK_DataSize       , SHIFT_DataSize);}

      bool     RawDataBlockHeader::getMonteCarloFlag() const {
        if(BITFIELDGET(RAWBUFF32[WORD_MonteCarloFlag]    , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag))
            return true;
        return false;
    }
      bool     RawDataBlockHeader::isCRCCalculationUsed() const {
        if(BITFIELDGET(RAWBUFF32[WORD_CRCCalculationUsed], MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed))
            return true;
        return false;
    }
      bool     RawDataBlockHeader::isConnectionInitialization() const {
        if(BITFIELDGET(RAWBUFF32[WORD_ConnectionInitialization], MASK_ConnectionInitialization, SHIFT_ConnectionInitialization))
            return true;
        return false;
    }
      bool     RawDataBlockHeader::isCloseConnection() const {
        if(BITFIELDGET(RAWBUFF32[WORD_CloseConnection]         , MASK_CloseConnection         , SHIFT_CloseConnection))
            return true;
        return false;
    }
      bool     RawDataBlockHeader::isMissingData() const {
        if(BITFIELDGET(RAWBUFF32[WORD_MissingData]             , MASK_MissingData             , SHIFT_MissingData))
            return true;
        return false;
    }



uint64_t RawDataBlockHeader::getTrigNum() const {
      uint64_t trigstamp;
      uint32_t low, high;

      low  = this->getTrigNumLo();
      high = this->getTrigNumHi();

      trigstamp=(uint64_t)high<<32 | low;
      return trigstamp;
}

//------------------------------------------------------------------------------
//Setters
      bool RawDataBlockHeader::setVersion  (uint32_t w){
          BITFIELDSET(RAWBUFF32[WORD_Version]       , w, MASK_Version       , SHIFT_Version);
          return true;
      }
      bool RawDataBlockHeader::setMarker   (uint32_t w){
          BITFIELDSET(RAWBUFF32[WORD_Marker]         , w, MASK_Marker          , SHIFT_Marker);
          return true;
      }
      bool RawDataBlockHeader::setTrigNumLo (uint32_t w){
          BITFIELDSET(RAWBUFF32[WORD_TriggerNumberLo], w, MASK_TriggerNumberLo, SHIFT_TriggerNumberLo);
          return true;
      }
      bool RawDataBlockHeader::setTrigNumHi (uint32_t w){
          BITFIELDSET(RAWBUFF32[WORD_TriggerNumberHi], w, MASK_TriggerNumberHi, SHIFT_TriggerNumberHi);
          return true;
      }
      bool RawDataBlockHeader::setNumMicroBlocks(uint16_t w){
          BITFIELDSET(RAWBUFF32[WORD_NumMicroBlocks] , w, MASK_NumMicroBlocks, SHIFT_NumMicroBlocks);
          return true;
      }
      bool RawDataBlockHeader::setBuffId   (uint16_t w){
          BITFIELDSET(RAWBUFF32[WORD_BuffId]         , w, MASK_BuffId        , SHIFT_BuffId);
          return true;
      }
      bool RawDataBlockHeader::setMissingData (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_MissingData]       , 0x1 , MASK_MissingData       , SHIFT_MissingData);
        else  BITFIELDSET(RAWBUFF32[WORD_MissingData]       , 0x0 , MASK_MissingData       , SHIFT_MissingData);
        return true;
      }
      bool RawDataBlockHeader::setMonteCarloFlag (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_MonteCarloFlag]    , 0x1 , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag);
        else  BITFIELDSET(RAWBUFF32[WORD_MonteCarloFlag]    , 0x0 , MASK_MonteCarloFlag    , SHIFT_MonteCarloFlag);
        return true;
      }
      bool RawDataBlockHeader::setCRCCalculationUsed (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_CRCCalculationUsed], 0x1 , MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed);
        else  BITFIELDSET(RAWBUFF32[WORD_CRCCalculationUsed], 0x0 , MASK_CRCCalculationUsed, SHIFT_CRCCalculationUsed);
        return true;
      }
      bool RawDataBlockHeader::setConnectionInitialization (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_ConnectionInitialization], 0x1 , MASK_ConnectionInitialization, SHIFT_ConnectionInitialization);
        else  BITFIELDSET(RAWBUFF32[WORD_ConnectionInitialization], 0x0 , MASK_ConnectionInitialization, SHIFT_ConnectionInitialization);
        return true;
      }
      bool RawDataBlockHeader::setCloseConnection (bool tf){
        if(tf)BITFIELDSET(RAWBUFF32[WORD_CloseConnection], 0x1 , MASK_CloseConnection, SHIFT_CloseConnection);
        else  BITFIELDSET(RAWBUFF32[WORD_CloseConnection], 0x0 , MASK_CloseConnection, SHIFT_CloseConnection);
        return true;
      }

      bool RawDataBlockHeader::setTrigNum(uint64_t trigstamp) {
             uint32_t low, high;

            low  = trigstamp & 0xffffffff;
            high = trigstamp >> 32;

            this->setTrigNumHi(high);
            this->setTrigNumLo(low);

            return true;
      }





      bool RawDataBlockHeader::setDataSize (uint32_t w){
          BITFIELDSET(RAWBUFF32[WORD_DataSize]       , w, MASK_DataSize      , SHIFT_DataSize);
          return true;
      }
      bool RawDataBlockHeader::addDataSize (uint32_t size){ return this->setDataSize      (this->getDataSize() + size);}
      bool RawDataBlockHeader::advanceNumMicroBlocks()    { return this->setNumMicroBlocks(this->getNumMicroBlocks() + 1);}


      bool RawDataBlockHeader::setBufferSource(void* pointer){
        _Buffer = pointer;
        return true;
      }
    bool RawDataBlockHeader::resetBufferSource(){
        this->setBufferSource(&_InternalBuffer[0]);
        return true;
    }

// clears all data in the buffer
    bool RawDataBlockHeader::clear(){
        int max_data = this->sizeofdata();

        // Loop over the buffer and zero it
        for(int i=0; i< max_data; ++i)
            RAWBUFF32[i]=0;

        return true;
    }
    bool RawDataBlockHeader::reset(){
        this->resetBufferSource();
        this->clear();
        return true;
    }
    bool RawDataBlockHeader::isInternalBuffer() const {

        if((this->_Buffer) == (void*)(&_InternalBuffer[0]))
            return true;
        return false;
  }


//------------------------------------------------------------------------------
// Print the Data Block Header
void RawDataBlockHeader::print() const{
    printf("DataBlock Header:  ");

    printf(" Version=%u"                   , this->getVersion());
    printf(" Marker=%u"                    , this->getMarker());
    printf(" TriggerNumberLo=%u"           , this->getTrigNumLo());
    printf(" TriggerNumberHi=%u"           , this->getTrigNumHi());
    printf(" NumMicroBlocks=%u"            , this->getNumMicroBlocks());
    printf(" BuffId=%u"                    , this->getBuffId());
    printf(" DataSize=%u"                  , this->getDataSize());
    printf(" MonteCarloFlag=%u"            , this->getMonteCarloFlag());
    printf(" isCRCCalculationUsed=%u"      , this->isCRCCalculationUsed());
    printf(" isConnectionInitialization=%u", this->isConnectionInitialization());
    printf(" isCloseConnection=%u"         , this->isCloseConnection());
    printf(" isMissingData=%u"             , this->isMissingData());
    printf("\n");

            
    std::cout<<"\nRawDataBlockHeader RAW 32bit words (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf(" ");
        this->printWord(i,false);
    }

    std::cout<<"\nRawDataBlockHeader RAW bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < this->sizeofdata(); ++i){
        printf("\n");
        this->printWord(i,true);
    }




}

//------------------------------------------------------------------------------
// Size of _Buffer in words
uint32_t RawDataBlockHeader::sizeofdata()const {return RAWDATABLOCKHEADER_SIZE;}
